package com.perforce.workshop.tjuricek.p4oauth.test_server;

import org.apache.commons.io.IOUtils;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.utils.URLEncodedUtils;
import org.apache.http.conn.ClientConnectionManager;
import org.apache.http.conn.scheme.Scheme;
import org.apache.http.conn.scheme.SchemeRegistry;
import org.apache.http.conn.ssl.*;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.impl.conn.SingleClientConnManager;
import org.eclipse.jetty.server.Request;
import org.eclipse.jetty.server.handler.AbstractHandler;

import javax.net.ssl.*;
import javax.net.ssl.SSLSocketFactory;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.net.*;
import java.security.KeyManagementException;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.List;

import static org.apache.commons.io.IOUtils.closeQuietly;

/**
 * Sets up our authorization header and redirects to our fetch page (I think)
 */
public class FetcherHelper extends AbstractHandler {

    /**
     * @param target
     * @param baseRequest
     * @param request
     * @param response
     * @throws IOException
     * @throws ServletException
     */
    @Override
    public void handle(String target, Request baseRequest, HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        if (baseRequest.getMethod().equals("GET") &&
                baseRequest.getPathInfo().equals("/fetcher")) {
            baseRequest.setHandled(true);

            String clientId = request.getParameter("client_id");
            String redirectUri = Conventions.getRedirectUri(Conventions.getHostname(), 8444);
            String accessToken = request.getParameter("access_token");

            String p4oauthUrl = String.format("https://%s:%d/p4token?redirect_uri=%s&client_id=%s",
                    Conventions.getHostname(), 8443,
                    URLEncoder.encode(redirectUri, "utf8"),
                    clientId);

            SSLContextBuilder sslContextBuilder = new SSLContextBuilder();
            SSLConnectionSocketFactory socketFactory = null;
            try {
                sslContextBuilder.loadTrustMaterial(null, new TrustSelfSignedStrategy());
                socketFactory = new SSLConnectionSocketFactory(sslContextBuilder.build(), SSLConnectionSocketFactory.ALLOW_ALL_HOSTNAME_VERIFIER);
            } catch (NoSuchAlgorithmException e) {
                e.printStackTrace();
            } catch (KeyStoreException e) {
                e.printStackTrace();
            } catch (KeyManagementException e) {
                e.printStackTrace();
            }
            CloseableHttpClient client = HttpClients.custom()
                    .setSSLSocketFactory(socketFactory)
                    .build();

            HttpGet httpGet = new HttpGet(p4oauthUrl);
            httpGet.setHeader("Authorization", "Bearer " + accessToken);
            CloseableHttpResponse httpResponse = client.execute(httpGet);
            try {
                if (httpResponse.getStatusLine().getStatusCode() == 200) {
                    String token = IOUtils.toString(httpResponse.getEntity().getContent(), "utf8");
                    response.sendRedirect(redirectUri + "?token=" + token);
                } else {
                    response.sendError(500, "oops");
                }
            } finally {
                closeQuietly(httpResponse);
                closeQuietly(client);
            }
        }
    }
}
