package com.perforce.workshop.tjuricek.p4oauth.storage;

import com.perforce.workshop.tjuricek.p4oauth.config.P4OAuthConfig;
import com.perforce.workshop.tjuricek.p4oauth.models.AccessToken;

import java.io.File;
import java.io.IOException;
import java.util.Optional;

/**
 */
public class AccessTokenStore implements FilesystemStore {

    private P4OAuthConfig config;

    public AccessTokenStore(P4OAuthConfig config) {
        this.config = config;
    }

    @Override
    public P4OAuthConfig getConfig() {
        return config;
    }

    public File getTokenFile(String token) {
        return getFile("tokens", token);
    }

    public Optional<AccessToken> read(String code) throws IOException {
        File tokenPath = getTokenFile(code);
        if (tokenPath.exists()) {
            return Optional.of(readYamlFile(AccessToken.class, tokenPath));
        } else {
            return Optional.empty();
        }
    }

    public void delete(String token) throws IOException {
        File tokenPath = getTokenFile(token);
        if (tokenPath.exists()) {
            tokenPath.delete();
        }
    }

    public void update(AccessToken accessToken) throws IOException {
        File tokenPath = getTokenFile(accessToken.getId());
        ensureParentExists(tokenPath);
        writeYamlFile(accessToken, tokenPath);
    }
}
